@@ -1,8 +1,6 @@
 """Img2dataset"""
 
-from multiprocessing import Pool
 from typing import List, Optional
-from tqdm import tqdm
 import fire
 import logging
 from .logger import LoggerProcess
@@ -10,9 +8,11 @@ from .resizer import Resizer
 from .writer import WebDatasetSampleWriter, FilesSampleWriter, ParquetSampleWriter
 from .reader import Reader
 from .downloader import Downloader
+from .distributor import multiprocessing_distributor, pyspark_distributor
 import fsspec
 import sys
 import signal
+import os
 
 logging.getLogger("exifread").setLevel(level=logging.CRITICAL)
 
@@ -41,13 +41,34 @@ def download(
     wandb_project: str = "img2dataset",
     oom_shard_count: int = 5,
     compute_md5: bool = True,
+    distributor: str = "multiprocessing",
+    subjob_size: int = 1000,
 ):
     """Download is the main entry point of img2dataset, it uses multiple processes and download multiple files"""
     config_parameters = dict(locals())
+
+    def make_path_absolute(path):
+        fs, p = fsspec.core.url_to_fs(path)
+        if fs.protocol == "file":
+            return os.path.abspath(p)
+        return path
+
+    output_folder = make_path_absolute(output_folder)
+    url_list = make_path_absolute(url_list)
+
     logger_process = LoggerProcess(output_folder, enable_wandb, wandb_project, config_parameters, processes_count)
     logger_process.start()
 
+    tmp_path = output_folder + "/_tmp"
+    fs, tmp_dir = fsspec.core.url_to_fs(tmp_path)
+    if not fs.exists(tmp_dir):
+        fs.mkdir(tmp_dir)
+
     def signal_handler(signal_arg, frame):  # pylint: disable=unused-argument
+        try:
+            fs.rm(tmp_dir, recursive=True)
+        except Exception as _:  # pylint: disable=broad-except
+            pass
         logger_process.terminate()
         sys.exit(0)
 
@@ -61,14 +82,23 @@ def download(
         fs.mkdir(output_path)
         start_shard_id = 0
     else:
-        existing_top_level_files = fs.glob(output_path + "/*")
+        existing_top_level_files = [x for x in fs.glob(output_path + "/*") if x != tmp_path]
         if len(existing_top_level_files) == 0:
             start_shard_id = 0
         else:
-            start_shard_id = max([int(x.split("/")[-1].split(".")[0]) for x in existing_top_level_files]) + 1
+            start_shard_id = (
+                max([int(x.split("/")[-1].split(".")[0]) for x in existing_top_level_files if x != tmp_path]) + 1
+            )
 
     reader = Reader(
-        url_list, input_format, url_col, caption_col, save_additional_columns, number_sample_per_shard, start_shard_id
+        url_list,
+        input_format,
+        url_col,
+        caption_col,
+        save_additional_columns,
+        number_sample_per_shard,
+        start_shard_id,
+        tmp_path,
     )
 
     if output_format == "webdataset":
@@ -103,14 +133,18 @@ def download(
     )
 
     print("Starting the downloading of this file")
-    with Pool(processes_count, maxtasksperchild=5) as process_pool:
-        for _ in tqdm(process_pool.imap_unordered(downloader, reader),):
-            pass
+    if distributor == "multiprocessing":
+        distributor_fn = multiprocessing_distributor
+    elif distributor == "pyspark":
+        distributor_fn = pyspark_distributor
+    else:
+        raise ValueError(f"Distributor {distributor} not supported")
 
-        process_pool.terminate()
-        process_pool.join()
-        del process_pool
+    distributor_fn(
+        processes_count, downloader, reader, subjob_size,
+    )
     logger_process.join()
+    fs.rm(tmp_dir, recursive=True)
 
 
 def main():
